@@ -47,7 +47,8 @@ namespace jsoncons {
         illegal_codepoint,
         illegal_surrogate_value,
         unpaired_high_surrogate,
-        illegal_unicode_character
+        illegal_unicode_character,
+        unexpected_character
     };
 
     class json_error_category_impl
@@ -122,21 +123,21 @@ namespace jsoncons {
                     return "Expected low surrogate following the high surrogate";
                 case json_errc::illegal_unicode_character:
                     return "Illegal unicode character";
+                case json_errc::unexpected_character:
+                    return "Unexpected character";
                 default:
                     return "Unknown JSON parser error";
                 }
         }
     };
 
-    inline
-    const std::error_category& json_error_category()
+    inline const std::error_category& json_error_category() noexcept
     {
       static json_error_category_impl instance;
       return instance;
     }
 
-    inline 
-    std::error_code make_error_code(json_errc result)
+    inline std::error_code make_error_code(json_errc result) noexcept
     {
         return std::error_code(static_cast<int>(result),json_error_category());
     }
