@@ -17,7 +17,7 @@
 #include <jsoncons/item_event_visitor.hpp>
 #include <jsoncons/json_exception.hpp>
 #include <jsoncons/json_visitor.hpp>
-#include <jsoncons/ser_context.hpp>
+#include <jsoncons/ser_util.hpp>
 #include <jsoncons/source.hpp>
 #include <jsoncons/staj_cursor.hpp>
 
@@ -55,9 +55,9 @@ public:
           cursor_handler_adaptor_(cursor_visitor_, alloc)
     {
         parser_.cursor_mode(true);
-        if (!done())
+        if (!parser_.done())
         {
-            next();
+            read_next();
         }
     }
 
@@ -93,9 +93,9 @@ public:
          eof_(false)
     {
         parser_.cursor_mode(true);
-        if (!done())
+        if (!parser_.done())
         {
-            next(ec);
+            read_next(ec);
         }
     }
 
@@ -110,9 +110,9 @@ public:
         cursor_visitor_.reset();
         cursor_handler_adaptor_.reset();
         eof_ = false;
-        if (!done())
+        if (!read_done())
         {
-            next();
+            read_next();
         }
     }
 
@@ -123,9 +123,9 @@ public:
         cursor_visitor_.reset();
         cursor_handler_adaptor_.reset();
         eof_ = false;
-        if (!done())
+        if (!read_done())
         {
-            next();
+            read_next();
         }
     }
 
@@ -135,9 +135,9 @@ public:
         cursor_visitor_.reset();
         cursor_handler_adaptor_.reset();
         eof_ = false;
-        if (!done())
+        if (!read_done())
         {
-            next(ec);
+            read_next(ec);
         }
     }
 
@@ -148,15 +148,15 @@ public:
         cursor_visitor_.reset();
         cursor_handler_adaptor_.reset();
         eof_ = false;
-        if (!done())
+        if (!read_done())
         {
-            next(ec);
+            read_next(ec);
         }
     }
 
     bool done() const override
     {
-        return parser_.done();
+        return read_done();
     }
 
     const staj_event& current() const override
@@ -206,12 +206,7 @@ public:
 
     void next() override
     {
-        std::error_code ec;
-        next(ec);
-        if (JSONCONS_UNLIKELY(ec))
-        {
-            JSONCONS_THROW(ser_error(ec,parser_.line(),parser_.column()));
-        }
+        read_next();
     }
 
     void next(std::error_code& ec) override
@@ -246,6 +241,22 @@ public:
         return staj_filter_view(cursor, pred);
     }
 private:
+
+    bool read_done() const
+    {
+        return parser_.done();
+    }
+
+    void read_next()
+    {
+        std::error_code ec;
+        read_next(ec);
+        if (JSONCONS_UNLIKELY(ec))
+        {
+            JSONCONS_THROW(ser_error(ec,parser_.line(),parser_.column()));
+        }
+    }
+
     void read_next(std::error_code& ec)
     {
         if (cursor_visitor_.in_available())
