@@ -97,7 +97,7 @@ TEST_CASE("json::as<jsoncons::bigint>()")
     {
         std::string s = "-18446744073709551617";
         jsoncons::json doc(s,  jsoncons::semantic_tag::bigint);
-        CHECK(bool(doc.as<jsoncons::bigint>() == jsoncons::bigint::parse(s)));
+        CHECK(bool(doc.as<jsoncons::bigint>() == jsoncons::bigint(s)));
     }
 }
 
@@ -109,18 +109,18 @@ TEST_CASE("json::as<__int128>()")
     std::string s1 = "-18446744073709551617";
 
     __int128 n{};
-    auto result = jsoncons::utility::dec_to_integer(s1.data(),s1.size(), n);
-    REQUIRE(result.ec == std::errc());
+    auto result = jsoncons::dec_to_integer(s1.data(),s1.size(), n);
+    REQUIRE(result.ec == std::errc{});
 
     jsoncons::json doc(s1);
 
     __int128 val = doc.as<__int128>();
 
     std::string s2;
-    jsoncons::utility::from_integer(val, s2);
+    jsoncons::from_integer(val, s2);
 
     std::string s3;
-    jsoncons::utility::from_integer(n, s3);
+    jsoncons::from_integer(n, s3);
 
     CHECK((n == val));
 }
@@ -131,17 +131,17 @@ TEST_CASE("json::as<unsigned __int128>()")
 
     unsigned __int128 n{};
 
-    auto result = jsoncons::utility::dec_to_integer(s1.data(),s1.size(), n);
-    REQUIRE(result.ec == std::errc());
+    auto result = jsoncons::dec_to_integer(s1.data(),s1.size(), n);
+    REQUIRE(result.ec == std::errc{});
 
     jsoncons::json doc(s1);
 
     unsigned __int128 val = doc.as<unsigned __int128>();
 
     std::string s2;
-    jsoncons::utility::from_integer(val, s2);
+    jsoncons::from_integer(val, s2);
     std::string s3;
-    jsoncons::utility::from_integer(n, s3);
+    jsoncons::from_integer(n, s3);
 
     CHECK((n == val));
 }
