@@ -1,8 +1,10 @@
 #define CATCH_CONFIG_MAIN
 
-#include <jsoncons/config/jsoncons_config.hpp>
+#include <jsoncons/config/compiler_support.hpp>
 #include <catch/catch.hpp>
 
+#include <iostream>
+
 TEST_CASE("configuration")
 {
     #if defined(__clang__) 
