@@ -8,6 +8,7 @@
 #include <jsoncons_ext/csv/csv.hpp>
 #include <jsoncons/json.hpp>
 
+#include <iostream>
 #include <vector>
 #include <utility>
 #include <ctime>
@@ -110,7 +111,7 @@ TEST_CASE("oss-fuzz issues")
     {
         std::string s = "-6.6E6";
         int64_t n{ 0 };
-        auto result = jsoncons::detail::to_integer(s.data(),s.size(), n);
+        auto result = jsoncons::to_integer(s.data(),s.size(), n);
         CHECK_FALSE(result); //-V521
     }
 
@@ -798,7 +799,6 @@ TEST_CASE("Fuzz target: fuzz_cbor_encoder")
         CHECK_FALSE(ec);
         //std::cout << visitor.get_result() << "" << '\n';
     }
-#endif
 
     // Fuzz target: fuzz_msgpack_encoder
     // Issue: Integer-overflow
@@ -817,6 +817,47 @@ TEST_CASE("Fuzz target: fuzz_cbor_encoder")
 
         REQUIRE_NOTHROW(reader.read(ec));
     }
+
+    // Fuzz target: fuzz_json_parser_max
+    // Issue: Timeout (exceeds 60 secs)
+    SECTION("issue 416794751")
+    {
+        std::string pathname = "fuzz_regression/input/clusterfuzz-testcase-minimized-fuzz_json_parser_max-4672808494104576";
+
+        std::ifstream is(pathname, std::ios_base::in | std::ios_base::binary);
+        CHECK(is); //-V521
+
+        std::string s2;
+        json_string_encoder visitor(s2);
+
+        json_stream_reader reader(is, visitor);
+        std::error_code ec;
+        reader.read(ec);
+        //std::cout << ec.message() << "\n";
+    }
+    // Fuzz target: fuzz_bigint
+    // Issue: Timeout (exceeds 60 secs)
+    SECTION("issue 416794751")
+    {
+        std::string pathname = "fuzz_regression/input/clusterfuzz-testcase-minimized-fuzz_bigint-5516555002904576";
+
+        std::ifstream is(pathname, std::ios_base::in | std::ios_base::binary);
+        CHECK(is); //-V521
+
+        std::istreambuf_iterator<char> eos;
+        std::string s(std::istreambuf_iterator<char>(is), eos);
+
+        std::cout << s.length() << "\n";
+
+        try {
+            bigint a("56654250564056135415631554531554513813");
+            bigint b(s);
+            bigint c = a % b;
+            bigint d = b % a;
+        }
+        catch (const std::runtime_error&) {}
+    }
+#endif
 }
 
 
